@@ -70,7 +70,9 @@ def checkout(
             src_path = entry.path
         else:
             assert entry.hash_info
-            odb = entry.odb or entry.cache or entry.remote
+            odb = index.odb_map.get(entry.key) or index.remote_map.get(
+                entry.key
+            )
             assert odb
             src_fs = odb.fs
             src_path = odb.oid_to_path(entry.hash_info.value)
@@ -113,8 +115,10 @@ def _get_changes(
         if change.typ == ADD:
             create.append(change.new)
         elif change.typ == MODIFY:
-            create.append(change.new)
-            delete.append(change.old)
+            if change.new:
+                create.append(change.new)
+            if change.old:
+                delete.append(change.old)
         elif change.typ == DELETE and delete:
             delete.append(change.old)
     return create, delete
